--[[

Broker_Reagents, a reagent tracker for World of Warcraft.
Copyright (C) 2009 Rabbit

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

local ldb = LibStub("LibDataBroker-1.1")
local PT = LibStub("LibPeriodicTable-3.1")

local track = {}
local trackAsOne = {}
local max = {}
local icons = {}
local colors = {}

local _, playerClass = UnitClass("player")
local reagentCount = {}
local reagentNames = {}

local additionalSets = {
	WARLOCK = {"Consumable.Warlock.Healthstone", "Consumable.Warlock.Spellstone", "Consumable.Warlock.Soulstone", "Consumable.Warlock.Firestone"},
	ROGUE = {"Consumable.Potion.Other.Energy", "Consumable.Weapon Buff.Poison.Anesthetic", "Consumable.Weapon Buff.Poison.Wound", "Consumable.Weapon Buff.Poison.Mind Numbing", "Consumable.Weapon Buff.Poison.Instant", "Consumable.Weapon Buff.Poison.Crippling", "Consumable.Weapon Buff.Poison.Deadly"},
	MAGE = {"Consumable.Mage.Mana Stone"},
	HUNTER = {"Misc.Reagent.Ammo.Bullet", "Misc.Reagent.Ammo.Arrow"},
	WARRIOR = {"Misc.Reagent.Ammo.Bullet", "Misc.Reagent.Ammo.Arrow"},
}

local classIcons = {
	DRUID = "Interface\\Icons\\INV_Misc_Branch_01",
	MAGE = "Interface\\Icons\\INV_Misc_Dust_01",
	PALADIN = "Interface\\Icons\\INV_Stone_WeightStone_05",
	PRIEST = "Interface\\Icons\\INV_Misc_Candle_03",
	ROGUE = "Interface\\Icons\\Trade_BrewPoison",
	SHAMAN = "Interface\\Icons\\INV_Jewelry_Talisman_06",
	WARLOCK = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02",
	DEATHKNIGHT = "Interface\\Icons\\INV_Misc_Dust_02",
	HUNTER = "Interface\\Icons\\INV_Ammo_Arrow_02",
	WARRIOR = "Interface\\Icons\\Misc_Book_09",
}

local ptClassTranslationTable = {
	DRUID = "Druid",
	MAGE = "Mage",
	PALADIN = "Paladin",
	PRIEST = "Priest",
	ROGUE = "Rogue",
	SHAMAN = "Shaman",
	WARLOCK = "Warlock",
	DEATHKNIGHT = "Death Knight",
	HUNTER = "Hunter",
	WARRIOR = "Warrior",
}

local qualityColor = {
	"|c009d9d9d",
	"|c00ffffff",
	"|c001eff00",
	"|c000070dd",
	"|c00a335ee",
	"|c00ff8000",
	"|c00e6cc80",
}

local bReagents = ldb:NewDataObject("Broker_Reagents", {
	type = "data source",
	icon = classIcons[playerClass],
	text = "OK",
})

local addon = CreateFrame("Frame")
local needsUpdate = true -- Force update on login, even though we know that BAG_UPDATE fires on login

local function update()
	for i in ipairs(reagentNames) do reagentNames[i] = nil end

	for i, v in ipairs(track) do
		local name, link, rarity, _, _, _, _, m, _, icon = GetItemInfo(v)
		if name then
			table.insert(reagentNames, name)
			reagentCount[name] = GetItemCount(link)
			max[name] = m
			icons[name] = icon
			colors[name] = qualityColor[rarity + 1]
		end
	end

	for setName, setItems in pairs(trackAsOne) do
		local counter = 0
		local stackSize = 0
		local setIcon = nil
		local setRarity = 0
		local properName = nil
		for k, v in pairs(setItems) do
			local name, link, rarity, _, _, _, _, m, _, icon = GetItemInfo(k)
			if name then
				if m > stackSize then stackSize = m end
				if rarity > setRarity then setRarity = rarity end
				if not properName then properName = name end
				counter = counter + GetItemCount(link)
				setIcon = icon
			end
		end
		if properName and counter > 0 then
			table.insert(reagentNames, properName)
			reagentCount[properName] = counter
			max[properName] = stackSize
			icons[properName] = setIcon
			colors[properName] = qualityColor[setRarity + 1]
		end
	end

	table.sort(reagentNames)
	
	local ok = true
	for i, v in ipairs(reagentNames) do
		local num = reagentCount[v]
		local stack = max[v]
		if num < stack then
			ok = nil
			break
		end
	end
	if ok then
		bReagents.text = "OK"
	else
		bReagents.text = "!"
	end

	needsUpdate = nil
end
function addon:BAG_UPDATE()
	needsUpdate = true
end
function addon:PLAYER_LOGIN()
	local class = ptClassTranslationTable[playerClass]

	local ptSet = PT:GetSetTable("Misc.Reagent.Class."..class)
	if type(ptSet) == "table" then
		for k, v in pairs(ptSet) do
			table.insert(track, k)
		end
	end

	if type(additionalSets[playerClass]) == "table" then
		for i, v in ipairs(additionalSets[playerClass]) do
			trackAsOne[v] = PT:GetSetTable(v)
		end
	end
end
do
	local total = 0
	addon:SetScript("OnUpdate", function(self, elapsed)
		total = total + elapsed
		if total > 5 then
			if needsUpdate then update() end
			total = 0
		end
	end)
end
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("BAG_UPDATE")
addon:RegisterEvent("PLAYER_LOGIN")

do
	local nameFormat = "|T%s:16|t %s%s|r"
	local countFormat = "|c00%s%d|r"
	function bReagents.OnTooltipShow(tt)
		for i, v in ipairs(reagentNames) do
			local n = nameFormat:format(icons[v], colors[v], v)
			local color = reagentCount[v] < max[v] and "ff0000" or "00ff00"
			local c = countFormat:format(color, reagentCount[v])
			tt:AddDoubleLine(n, c)
		end
	end
end

